@@ -38,9 +38,6 @@ char **cmdargv = NULL;
 int cmdargc = 0;
 char *mode = NULL;
 
-struct termios old_term_settings;
-int control_via_stdin = 0;
-
 int main(int argc, char **argv)
 {
 	atexit(feh_clean_exit);
@@ -76,11 +73,14 @@ int main(int argc, char **argv)
 		feh_wm_set_bg_filelist(opt.bgmode);
 		exit(0);
 	}
-	else {
+	else if (opt.display){
 		/* Slideshow mode is the default. Because it's spiffy */
 		opt.slideshow = 1;
 		init_slideshow_mode();
 	}
+	else {
+		eprintf("Invalid option combination");
+	}
 
 	/* main event loop */
 	while (feh_main_iteration(1));
@@ -122,19 +122,7 @@ int feh_main_iteration(int block)
 		 *   also don't get input from stdin anyway.
 		 */
 		if (isatty(STDIN_FILENO) && !opt.multiwindow && getpgrp() == (tcgetpgrp(STDIN_FILENO))) {
-			control_via_stdin = 1;
-			struct termios ctrl;
-			if (tcgetattr(STDIN_FILENO, &old_term_settings) == -1)
-				eprintf("tcgetattr failed");
-			if (tcgetattr(STDIN_FILENO, &ctrl) == -1)
-				eprintf("tcgetattr failed");
-			ctrl.c_iflag &= ~(PARMRK | ISTRIP
-					| INLCR | IGNCR | IXON);
-			ctrl.c_lflag &= ~(ECHO | ICANON | IEXTEN);
-			ctrl.c_cflag &= ~(CSIZE | PARENB);
-			ctrl.c_cflag |= CS8;
-			if (tcsetattr(STDIN_FILENO, TCSANOW, &ctrl) == -1)
-				eprintf("tcsetattr failed");
+			setup_stdin();
 		}
 	}
 
@@ -234,9 +222,16 @@ void feh_clean_exit(void)
 	if(disp)
 		XCloseDisplay(disp);
 
-	if (control_via_stdin && isatty(STDIN_FILENO))
-		if (tcsetattr(STDIN_FILENO, TCSANOW, &old_term_settings) == -1)
-			eprintf("tcsetattr failed");
+	/*
+	 * Only restore the old terminal settings if
+	 * - we changed them in the first place
+	 * - stdin still is a terminal (it might have been closed)
+	 * - stdin still belongs to us (we might have been detached from the
+	 *   controlling terminal, in that case we probably shouldn't be messing
+	 *   around with it) <https://github.com/derf/feh/issues/324>
+	 */
+	if (control_via_stdin && isatty(STDIN_FILENO) && getpgrp() == (tcgetpgrp(STDIN_FILENO)))
+		restore_stdin();
 
 	if (opt.filelistfile)
 		feh_write_filelist(filelist, opt.filelistfile);
