@@ -95,7 +95,7 @@ void feh_add_timer(void (*func) (void *data), void *data, double in, char *name)
 
 	D(("adding timer %s for %f seconds time\n", name, in));
 	feh_remove_timer(name);
-	ft = malloc(sizeof(_fehtimer));
+	ft = emalloc(sizeof(_fehtimer));
 	ft->next = NULL;
 	ft->func = func;
 	ft->data = data;
@@ -103,7 +103,6 @@ void feh_add_timer(void (*func) (void *data), void *data, double in, char *name)
 	ft->just_added = 1;
 	ft->in = in;
 	D(("ft->in = %f\n", ft->in));
-	tally = 0.0;
 	if (!first_timer) {
 		D(("No first timer\n"));
 		first_timer = ft;
