@@ -128,7 +128,7 @@ void winwidget_center_image(winwidget w);
 void winwidget_render_image(winwidget winwid, int resize, int force_alias);
 void winwidget_rotate_image(winwidget winid, double angle);
 void winwidget_move(winwidget winwid, int x, int y);
-void winwidget_resize(winwidget winwid, int w, int h);
+void winwidget_resize(winwidget winwid, int w, int h, int force_resize);
 void winwidget_setup_pixmaps(winwidget winwid);
 void winwidget_update_title(winwidget ret);
 void winwidget_update_caption(winwidget winwid);
@@ -141,8 +141,8 @@ void winwidget_get_geometry(winwidget winwid, int *rect);
 int winwidget_get_width(winwidget winwid);
 int winwidget_get_height(winwidget winwid);
 winwidget winwidget_get_from_window(Window win);
-winwidget winwidget_create_from_file(gib_list * filename, char *name, char type);
-winwidget winwidget_create_from_image(Imlib_Image im, char *name, char type);
+winwidget winwidget_create_from_file(gib_list * filename, char type);
+winwidget winwidget_create_from_image(Imlib_Image im, char type);
 void winwidget_rename(winwidget winwid, char *newname);
 void winwidget_destroy(winwidget winwid);
 void winwidget_create_window(winwidget ret, int w, int h);
